@@ -22,6 +22,9 @@ sizes = {
     'header': 8,
     'dictHeader': 8
 }
+codec_overrides = {
+    'ru_RU': 'cp1251',
+}
 
 class ACEFileException(Exception): #{{{1
     def __init__(self, msgsource, errmsg):
@@ -45,10 +48,17 @@ class ACEFileDecodeException(ACEFileException): #{{{1
 
 class ACEFile(): #{{{1
     _dicts = {}
-    _encoding = locale.getdefaultlocale()[1]
 
     def __init__(self): #{{{2
+        cur_locale = locale.getdefaultlocale()
+
+        if cur_locale[0] in codec_overrides:
+            self._encoding = codec_overrides[cur_locale[0]]
+        else:
+            self._encoding = cur_locale[1]
+
         self.read()
+
         return
     
     def _bytesToInt(self, bytes): #{{{2
