@@ -6,35 +6,82 @@ from subprocess    import call
 from PyQt4.QtCore  import Qt, QObject, QSettings, QString, QStringList, \
                           QVariant, SIGNAL, SLOT
 from PyQt4.QtGui   import QAbstractItemView, QComboBox, QDialog, \
-                          QDialogButtonBox, QHBoxLayout, QLabel, QLineEdit, \
-                          QListWidget, QMenuBar, QMessageBox, QPushButton, \
-                          QStringListModel, QVBoxLayout, QWidget
-from ACE_file      import ACEFile;
+                          QDialogButtonBox, QHBoxLayout, QInputDialog, \
+                          QLabel, QLineEdit, QListWidget, QMenuBar, \
+                          QMessageBox, QPushButton, QStringListModel, \
+                          QVBoxLayout, QWidget
+from ACE_file      import ACEFile
+from ACE_utils     import apply_blacklist
 
-version = '0.0.7'
+version = '0.0.8'
 
 langLookup = {
-    3: 'English (United Kingdom)',
-    4: 'Français (France)',
-    5: 'Deutsch (Deutschland)',
-    6: 'Español (España)',
-    8: 'Português (Portugal)',
-    9: 'Svenska (Sverige)',
-    10: 'Suomi (Suomi)',
-    11: 'Norsk (Norge)',
-    12: 'Dansk (Danmark)',
-    13: 'Nederlands (Nederland)',
-    14: 'Ελληνικά (Ελλάδα)',
-    22: 'Polski (Polska)',
-    23: 'Čeština (Česká republika)',
-    39: 'Русский (Россия)',
-    46: 'English (United States of America)',
-    47: 'Italiano (Italia)',
-    51: 'Español (América Latina)',
-    52: 'Français (Québec)',
+    3: QString.fromUtf8('English (United Kingdom)'),
+    4: QString.fromUtf8('Français (France)'),
+    5: QString.fromUtf8('Deutsch (Deutschland)'),
+    6: QString.fromUtf8('Español (España)'),
+    8: QString.fromUtf8('Português (Portugal)'),
+    9: QString.fromUtf8('Svenska (Sverige)'),
+    10: QString.fromUtf8('Suomi (Suomi)'),
+    11: QString.fromUtf8('Norsk (Norge)'),
+    12: QString.fromUtf8('Dansk (Danmark)'),
+    13: QString.fromUtf8('Nederlands (Nederland)'),
+    14: QString.fromUtf8('Ελληνικά (Ελλάδα)'),
+    22: QString.fromUtf8('Polski (Polska)'),
+    23: QString.fromUtf8('Čeština (Česká republika)'),
+    39: QString.fromUtf8('Русский (Россия)'),
+    46: QString.fromUtf8('English (United States of America)'),
+    47: QString.fromUtf8('Italiano (Italia)'),
+    51: QString.fromUtf8('Español (América Latina)'),
+    52: QString.fromUtf8('Français (Québec)'),
 }
 
-delModes = QStringList(('Normal', 'Confirm', 'OnClick'))
+delModes = QStringList(('Normal', 'Confirm', 'OnClick', 'Blacklist'))
+
+class ACESettings(): #{{{1
+    def __init__(self): #{{{2
+        self._settings = QSettings('Auto-Complete Editor', 'Auto-Complete Editor')
+
+    def getDelMode(self): #{{{2
+        return self._settings.value('DeleteMode').toString()
+
+    def getDisplayDictionary(self): #{{{2
+        return self._settings.value('displayDictionary').toInt()[0]
+
+    def getBlacklist(self): #{{{2
+        blacklist = {}
+
+        self._settings.beginGroup('blacklist')
+        if self._settings.contains('languages'):
+            s = self._settings.value('languages').toString()
+            langList = s.split(',')
+
+            for i in langList:
+                blacklist[int(i)] = self._settings.value(i).toStringList()
+
+        self._settings.endGroup()
+
+        return blacklist
+
+    def setDelMode(self, mode): #{{{2
+        self._settings.setValue('DeleteMode', mode)
+
+    def setDisplayDictionary(self, dict): #{{{2
+        self._settings.setValue('displayDictionary', dict)
+
+    def setBlacklist(self, langList, blacklist): #{{{2
+        self._settings.remove('blacklist')
+
+        if len(langList) == 0:
+            return
+
+        self._settings.beginGroup('blacklist')
+        self._settings.setValue('languages', ','.join(langList))
+
+        for i in langList:
+            self._settings.setValue(i, blacklist[int(i)])
+
+        self._settings.endGroup()
 
 class ACEAboutWindow(QDialog): #{{{1
     def __init__(self, parent): #{{{2
@@ -103,71 +150,32 @@ class ACEConfigWindow(QDialog): #{{{1
         QObject.connect(btns, SIGNAL('rejected()'), self, SLOT('reject()'))
         lyt.addWidget(btns)
 
-    def changeDeleteModeIndex(self, modeIndex): #{{{1
+    def changeDeleteModeIndex(self, modeIndex): #{{{2
         mode = delModes[modeIndex]
 
         self.changeDeleteMode(mode)
 
-    def changeDeleteMode(self, mode): #{{{1
+    def changeDeleteMode(self, mode): #{{{2
         if mode != self._parent.newDelMode:
             self._parent.newDelMode = mode
 
-class ACEMainWindow(QWidget): #{{{1
+class ACEViewWindow(QWidget): #{{{1
+    dicts = {}
     maemo = False
-    _dict  = None
-    _dicts = {}
 
-    def __init__(self): #{{{2
-        QWidget.__init__(self, None)
+    def __init__(self, parent): #{{{2
+        QWidget.__init__(self, parent)
 
-        try:
-            from PyQt4.QtMaemo5 import QMaemo5InformationBox
-            self.maemo = True
-            self.setAttribute(Qt.WA_Maemo5ShowProgressIndicator, True)
-        except:
-            pass
-
-        # Set to the N900 screen size to aid testing
-        self.setMinimumSize(800, 480)
-        self.setWindowTitle('Auto-Complete Editor')
-
-        self._settings = QSettings('Auto-Complete Editor', 'Auto-Complete Editor')
-        self.delMode = self._settings.value('DeleteMode').toString()
-        if self.delMode == '':
-            self.delMode = 'Normal'
-
-        self.newDelMode = self.delMode
-
-        QHBoxLayout(self)
+        if self.maemo:
+            self.setAttribute(Qt.WA_Maemo5StackedWindow);
 
     def _getLangName(self, langId): #{{{2
         if langId in langLookup:
-            return QString.fromUtf8(langLookup[langId])
+            return langLookup[langId]
         else:
             QMessageBox.information(self, 'Unknown dictionary', QString('An unknown dictionary ID has been found.\nPlease contact\n\tRobin Hill <maemo@robinhill.me.uk>\nwith:\n\t- the ID: %1\n\t- the dictionary language you have selected (from Settings/Text input)').arg(langId))
             return 'Unknown'
 
-    def about(self): #{{{2
-        aboutWin = ACEAboutWindow(self)
-        aboutWin.exec_()
-
-    def config(self): #{{{2
-        configWin = ACEConfigWindow(self)
-        if configWin.exec_() == QDialog.Rejected:
-            self.newDelMode = self.delMode
-
-        if self.newDelMode != self.delMode:
-            if self.delMode == 'OnClick':
-                self._delBtn.setEnabled(True)
-                QObject.disconnect(self._listView, SIGNAL('itemClicked(QListWidgetItem *)'), self.deleteEntry)
-
-            if self.newDelMode == 'OnClick':
-                self._delBtn.setEnabled(False)
-                QObject.connect(self._listView, SIGNAL('itemClicked(QListWidgetItem *)'), self.deleteEntry)
-
-            self.delMode = self.newDelMode
-            self._settings.setValue('DeleteMode', self.delMode)
-
     def addEntry(self, checked): #{{{2
         dlg = QDialog(self)
         dlg.setWindowTitle('Enter new value')
@@ -191,19 +199,11 @@ class ACEMainWindow(QWidget): #{{{1
 
         lwrText = newText.toLower()
 
-        if lwrText in self._dicts[self._curLang]:
+        if lwrText in self.dicts[self._curLang]:
             return
 
         self._listView.addItem(lwrText)
-        self._dicts[self._curLang].append(lwrText)
-
-    def clearEntries(self, checked): #{{{2
-        if self.delMode == 'Confirm':
-            if QMessageBox.question(self, 'Delete confirmation', 'Please confirm that you want to delete all entries', QMessageBox.Ok|QMessageBox.Cancel) != QMessageBox.Ok:
-                return
-
-        self._listView.clear()
-        self._dicts[self._curLang].clear()
+        self.dicts[self._curLang].append(lwrText)
 
     def deleteEntries(self, checked): #{{{2
         if self.delMode == 'Confirm':
@@ -218,7 +218,13 @@ class ACEMainWindow(QWidget): #{{{1
         self._listView.takeItem(row)
         val = entry.data(Qt.DisplayRole).toString()
 
-        self._dicts[self._curLang].removeAt(self._dicts[self._curLang].indexOf(val))
+        self.dicts[self._curLang].removeAt(self.dicts[self._curLang].indexOf(val))
+
+        if self.delMode == 'Blacklist':
+            if not self._curLang in self.blacklists:
+                self.blacklists[self._curLang] = QStringList()
+
+            self.blacklists[self._curLang].append(val)
         
     def loadDictionary(self, row): #{{{2
         self._listView.clear()
@@ -226,28 +232,162 @@ class ACEMainWindow(QWidget): #{{{1
         if row >= 0:
             (dictId, check) = self._dictList.itemData(row).toInt()
             self._curLang = dictId
-            self._listView.addItems(self._dicts[dictId])
-            self._settings.setValue('displayDictionary', dictId)
+            self._listView.addItems(self.dicts[dictId])
+
+    def setupScreen(self): #{{{2
+        self._dictList = QComboBox()
+        self._dictList.setInsertPolicy(QComboBox.NoInsert)
+        QObject.connect(self._dictList, SIGNAL('currentIndexChanged(int)'), self.loadDictionary)
+
+        self._listView = QListWidget()
+        self._listView.setUniformItemSizes(True)
+        self._listView.setSelectionMode(QAbstractItemView.MultiSelection)
+        self._listView.setSortingEnabled(True)
+
+        if self.delMode == 'OnClick':
+            QObject.connect(self._listView, SIGNAL('itemClicked(QListWidgetItem *)'), self.deleteEntry)
+
+        self._btnBox = QDialogButtonBox()
+        self._btnBox.setOrientation(Qt.Vertical)
+        btn = self._btnBox.addButton('Delete selected', QDialogButtonBox.ActionRole)
+        QObject.connect(btn, SIGNAL('clicked(bool)'), self.deleteEntries)
+        if self.delMode == 'OnClick':
+            btn.setEnabled(False)
+        self._delBtn = btn
+        btn = self._btnBox.addButton('Add entry', QDialogButtonBox.ActionRole)
+        QObject.connect(btn, SIGNAL('clicked(bool)'), self.addEntry)
+
+        lyt = QVBoxLayout()
+        lyt.addWidget(self._dictList)
+        lyt.addWidget(self._listView)
+
+        self.layout().addLayout(lyt)
+        self.layout().addWidget(self._btnBox)
+
+class ACEMainWindow(ACEViewWindow): #{{{1
+    _dict = None
+    blacklists = {}
+
+    def __init__(self): #{{{2
+        try:
+            from PyQt4.QtMaemo5 import QMaemo5InformationBox
+            self.maemo = True
+        except:
+            pass
+
+        ACEViewWindow.__init__(self, None)
+
+        if self.maemo:
+            self.setAttribute(Qt.WA_Maemo5ShowProgressIndicator, True)
+
+        self._blWindow = ACEBlacklistWindow(self)
+        self._blWindow.setWindowFlags(self._blWindow.windowFlags() | Qt.Window)
+        self._blWindow.setupScreen()
+
+        # Set to the N900 screen size to aid testing
+        self.setMinimumSize(800, 480)
+        self.setWindowTitle('Auto-Complete Editor')
+
+        self._settings = ACESettings()
+        self.delMode = self._settings.getDelMode()
+        if self.delMode == '':
+            self.delMode = 'Normal'
+
+        self.newDelMode = self.delMode
+
+        QHBoxLayout(self)
+
+    def about(self): #{{{2
+        aboutWin = ACEAboutWindow(self)
+        aboutWin.exec_()
+
+    def apply_blacklist(self): #{{{2
+        for i in self.dicts.keys():
+            if i in self.blacklists:
+                if apply_blacklist(self.dicts[i], self.blacklists[i]):
+                    if i == self._curLang:
+                        self._listView.clear()
+                        self._listView.addItems(self.dicts[i])
+
+    def edit_blacklist(self): #{{{2
+        if not self._curLang in self.blacklists:
+            self.blacklists[self._curLang] = QStringList()
+
+        self._blWindow.dicts = self.blacklists
+        self._blWindow.loadData(self._curLang)
+        self._blWindow.show()
+#        blacklistWin = ACEBlacklistWindow(self)
+#        blacklistWin.setupScreen()
+#        blacklistWin.loadData(self._curLang)
+
+#        blacklistWin.exec_()
+
+        self.saveBlacklists()
+
+    def config(self): #{{{2
+        configWin = ACEConfigWindow(self)
+        if configWin.exec_() == QDialog.Rejected:
+            self.newDelMode = self.delMode
+
+        if self.newDelMode != self.delMode:
+            if self.delMode == 'OnClick':
+                self._delBtn.setEnabled(True)
+                QObject.disconnect(self._listView, SIGNAL('itemClicked(QListWidgetItem *)'), self.deleteEntry)
+            elif self.delMode == 'Blacklist':
+                self._delBtn.setText('Delete')
+
+            if self.newDelMode == 'OnClick':
+                self._delBtn.setEnabled(False)
+                QObject.connect(self._listView, SIGNAL('itemClicked(QListWidgetItem *)'), self.deleteEntry)
+            elif self.newDelMode == 'Blacklist':
+                self._delBtn.setText('Blacklist')
+
+            self.delMode = self.newDelMode
+            self._settings.setDelMode(self.delMode);
+
+    def loadDictionary(self, row): #{{{2
+        ACEViewWindow.loadDictionary(self, row)
+
+        if row >= 0:
+            self._settings.setDisplayDictionary(self._curLang)
+
+    def clearEntries(self, checked): #{{{2
+        if self.delMode == 'Confirm':
+            if QMessageBox.question(self, 'Delete confirmation', 'Please confirm that you want to delete all entries', QMessageBox.Ok|QMessageBox.Cancel) != QMessageBox.Ok:
+                return
+
+        self._listView.clear()
+        self.dicts[self._curLang].clear()
 
     def revertChanges(self, checked): #{{{2
         self.loadData()
 
+    def saveBlacklists(self): #{{{2
+        blacklistLangs = []
+        for i in self.blacklists.keys():
+            if len(self.blacklists[i]) > 0:
+                blacklistLangs.append('%s' % i)
+
+        self._settings.setBlacklist(blacklistLangs, self.blacklists)
+
     def saveChanges(self, checked): #{{{2
         if self.maemo:
             self.setAttribute(Qt.WA_Maemo5ShowProgressIndicator, True)
 
+        self.saveBlacklists()
+
         langs = []
 
-        for i in self._dicts.keys():
-            if len(self._dicts[i]) > 0:
+        for i in self.dicts.keys():
+            if len(self.dicts[i]) > 0:
                 langs.append(i)
 
-        for i in self._dicts.keys():
+        for i in self.dicts.keys():
             if len(langs) == 0:
                 langs.append(i)
             
             if i in langs:
-                self._dict.setDict(i, self._dicts[i])
+                self._dict.setDict(i, self.dicts[i])
             else:
                 self._dict.deleteDict(i)
             
@@ -267,46 +407,24 @@ class ACEMainWindow(QWidget): #{{{1
 
         act = menu.addAction('Settings')
         QObject.connect(act, SIGNAL('triggered()'), self.config)
+        act = menu.addAction('Edit Blacklist')
+        QObject.connect(act, SIGNAL('triggered()'), self.edit_blacklist)
+        act = menu.addAction('Apply Blacklist')
+        QObject.connect(act, SIGNAL('triggered()'), self.apply_blacklist)
         act = menu.addAction('About')
         QObject.connect(act, SIGNAL('triggered()'), self.about)
         self.layout().setMenuBar(menu)
 
     def setupScreen(self): #{{{2
-        self._dictList = QComboBox()
-        self._dictList.setInsertPolicy(QComboBox.NoInsert)
-        QObject.connect(self._dictList, SIGNAL('currentIndexChanged(int)'), self.loadDictionary)
+        ACEViewWindow.setupScreen(self)
 
-        self._listView = QListWidget()
-        self._listView.setUniformItemSizes(True)
-        self._listView.setSelectionMode(QAbstractItemView.MultiSelection)
-        self._listView.setSortingEnabled(True)
-
-        if self.delMode == 'OnClick':
-            QObject.connect(self._listView, SIGNAL('itemClicked(QListWidgetItem *)'), self.deleteEntry)
-
-        btnBox = QDialogButtonBox()
-        btnBox.setOrientation(Qt.Vertical)
-        btn = btnBox.addButton('Delete all', QDialogButtonBox.ActionRole)
+        btn = self._btnBox.addButton('Delete all', QDialogButtonBox.ActionRole)
         QObject.connect(btn, SIGNAL('clicked(bool)'), self.clearEntries)
-        btn = btnBox.addButton('Delete selected', QDialogButtonBox.ActionRole)
-        QObject.connect(btn, SIGNAL('clicked(bool)'), self.deleteEntries)
-        if self.delMode == 'OnClick':
-            btn.setEnabled(False)
-        self._delBtn = btn
-        btn = btnBox.addButton('Add entry', QDialogButtonBox.ActionRole)
-        QObject.connect(btn, SIGNAL('clicked(bool)'), self.addEntry)
-        btn = btnBox.addButton('Save changes', QDialogButtonBox.AcceptRole)
+        btn = self._btnBox.addButton('Save changes', QDialogButtonBox.AcceptRole)
         QObject.connect(btn, SIGNAL('clicked(bool)'), self.saveChanges)
-        btn = btnBox.addButton('Revert changes', QDialogButtonBox.ResetRole)
+        btn = self._btnBox.addButton('Revert changes', QDialogButtonBox.ResetRole)
         QObject.connect(btn, SIGNAL('clicked(bool)'), self.revertChanges)
 
-        lyt = QVBoxLayout()
-        lyt.addWidget(self._dictList)
-        lyt.addWidget(self._listView)
-
-        self.layout().addLayout(lyt)
-        self.layout().addWidget(btnBox)
-
         # Clear the progress indicator
         if self.maemo:
             self.setAttribute(Qt.WA_Maemo5ShowProgressIndicator, False)
@@ -320,13 +438,13 @@ class ACEMainWindow(QWidget): #{{{1
         if self._dict == None:
             self._dict = ACEFile()
 
-        (curDict, check) = self._settings.value('displayDictionary').toInt()
+        curDict = self._settings.getDisplayDictionary()
         dictLangs = self._dict.getLanguages()
         self._dictList.clear()
-        self._dicts = {}
+        self.dicts = {}
 
         for lang in dictLangs:
-            self._dicts[lang] = self._dict.getDict(lang)
+            self.dicts[lang] = self._dict.getDict(lang)
             self._dictList.addItem(self._getLangName(lang), QVariant(lang))
 
         if curDict > 0 and curDict in dictLangs:
@@ -339,5 +457,96 @@ class ACEMainWindow(QWidget): #{{{1
         if len(dictLangs) == 1:
             self._dictList.setEnabled(False)
         
+        self.blacklists.clear()
+
+        self.blacklists = self._settings.getBlacklist()
+
         if updateSpinner:
             self.setAttribute(Qt.WA_Maemo5ShowProgressIndicator, False)
+
+class ACEBlacklistWindow(ACEViewWindow): #{{{1
+    def __init__(self, parent): #{{{2
+        self.maemo   = parent.maemo
+
+        ACEViewWindow.__init__(self, parent)
+
+        self._parent = parent
+        self.dicts   = parent.blacklists
+
+        self.setWindowTitle('Blacklist Editor')
+        self.delMode = 'Normal'
+
+        QHBoxLayout(self)
+
+    def setupScreen(self): #{{{2
+        ACEViewWindow.setupScreen(self)
+
+        btn = self._btnBox.addButton('Add dictionary', QDialogButtonBox.ActionRole)
+        QObject.connect(btn, SIGNAL('clicked(bool)'), self.addDictionary)
+        if len(self.dicts) >= len(self._parent.dicts):
+            btn.setEnabled(False)
+
+        btn = self._btnBox.addButton('Delete dictionary', QDialogButtonBox.ActionRole)
+        QObject.connect(btn, SIGNAL('clicked(bool)'), self.deleteDictionary)
+
+
+    def loadData(self, dictId = 0): #{{{2
+        updateSpinner = False
+        if self.maemo and not self.testAttribute(Qt.WA_Maemo5ShowProgressIndicator):
+            updateSpinner = True
+            self.setAttribute(Qt.WA_Maemo5ShowProgressIndicator, True)
+
+        dictLangs = self.dicts.keys()
+        self._dictList.clear()
+
+        if not dictId in dictLangs:
+            dictId = dictLangs[0]
+
+        for lang in dictLangs:
+            self._dictList.addItem(self._getLangName(lang), QVariant(lang))
+
+        self._dictList.setCurrentIndex(dictLangs.index(dictId))
+
+        if len(dictLangs) > 1:
+            self._dictList.setEnabled(True)
+        else:
+            self._dictList.setEnabled(False)
+
+        if updateSpinner:
+            self.setAttribute(Qt.WA_Maemo5ShowProgressIndicator, False)
+
+    def addDictionary(self, checked): #{{{2
+        dlg = QDialog(self)
+        dlg.setWindowTitle('Add dictionary')
+
+        cmbBox = QComboBox()
+        for lang in self._parent.dicts.keys():
+            if not lang in self.dicts:
+                cmbBox.addItem(self._getLangName(lang), QVariant(lang))
+
+        btnBox = QDialogButtonBox(QDialogButtonBox.Ok|QDialogButtonBox.Cancel)
+        QObject.connect(btnBox, SIGNAL('accepted()'), dlg, SLOT('accept()'))
+        QObject.connect(btnBox, SIGNAL('rejected()'), dlg, SLOT('reject()'))
+
+        lyt = QHBoxLayout(dlg)
+        lyt.addWidget(cmbBox)
+        lyt.addWidget(btnBox)
+
+        ok = dlg.exec_()
+
+        if not ok:
+            return
+
+        newDict = cmbBox.itemData(cmbBox.currentIndex()).toInt()[0]
+        self.dicts[newDict] = QStringList()
+        self.loadData(newDict)
+
+    def deleteDictionary(self, checked): #{{{2
+        dictLangs = self.dicts.keys()
+
+        if len(dictLangs) > 1:
+            del self.dicts[self._curLang]
+        else:
+            self.dicts[self._curLang].clear()
+
+        self.loadData()
